;==============================================================================
; Capture pixels
; Utilities to capture pixels from ZX Spectrum screen and make a XY table
;==============================================================================


;==============================================================================
; Constants
;==============================================================================



    ORG 25088
    JP Main

;==============================================================================
; Variables
;==============================================================================
; 25091
StartX
    DEFB 0                  ; Start X position
; 25092
StartY
    DEFB 0                  ; Start Y position
; 25093
EndX
    DEFB 255                ; End X position
; 25094
EndY
    DEFB 191                ; End Y position
; 25095
OutTable
    DEFW 32768              ; Address of output table

;==============================================================================
; Main program
;==============================================================================

Main
    LD IX,(OutTable)        ; IX is pointing to output table of XY coordinates
    LD A,(StartX)           ; Fetch X coordinate
    LD C,A
    DEC C                   ; X coordinate will be incremented below
Main.l0
    INC C                   ; Increment X coordinate
    LD A,(StartY)           ; Fetch Y coordinate
    LD B,A
    DEC B                   ; Y coordinate will be incremented below
Main.l1
    INC B                   ; Increment Y coordinate
    PUSH BC                 ; Save coordinates
    CALL VideoPixelAdd      ; Calculate screen address
    POP BC                  ; Restore coordinates
    LD A,E                  ; Test if pixel is on
    AND (HL)
    JR Z,Main.s0            ; Skip, if pixel was off
    LD (IX+0),C             ; Put X and Y coordinates into table
    LD (IX+1),B
    INC IX                  ; Advance table pointer by 2 bytes
    INC IX
    XOR A                   ; Test if pointer exceeds last RAM address, i.e.,
    OR IXh                  ;IX = 0
    JR Z,Main.s1            ; Jump to end of routine if it was exceeded
Main.s0
    LD A,(EndY)
    CP B
    JR NZ,Main.l1
    LD A,(EndX)             ; Test if X is equal to 
    CP C
    JR NZ,Main.l0
Main.s1
    PUSH IX                 ; The USR return value is the address after last
    POP BC                  ;byte of output table
    RET                     ; End of program


;==============================================================================
; Subroutines
;==============================================================================

;------------------------------------------------------------------------------
; Video Pixel Address (VideoPixelAdd)
; This subroutine accepts as parameters, a pair of coordinates defining a pixel 
; position, and it returns the corresponding address at video RAM and 1-bit 
; mask with the position of pixel within the byte.
;------------------------------------------------------------------------------
; Parameters:
;       B=vertical coordinate  (0-191)
;       C=horizontal coordinate (0-255)
; Output:
;       HL=address at video RAM
;       E=1-bit mask of pixel for byte pointed by HL
;       B is unchanged (vertical coordinate)
;       C=byte column position (0-31)
;       A=H
;       Flags: C=0; Z=0; S=0
;------------------------------------------------------------------------------

    INCLUDE "VideoRAMAddress.asm"



