;==============================================================================
; Demo Intro
; Code by TK90X Fan
; Gfx by Alone Coder, TK90X and Nitrofurano
; Music by nq
; Additional code by:
;   Einar Saukas (RCS & DZX7)
;   S.V.Bulba (PT3 player)
;==============================================================================

;==============================================================================
; Constants
;==============================================================================

CarColor            EQU %01000010   ; Flash 0, bright 1, paper 0, ink 2
AnimSize            EQU 1006        ; Max dot number for car anim

    ORG 24576

;------------------------------------------------------------------------------
; PT3 player and music module (24576-35012)
;------------------------------------------------------------------------------
pt3player
pt3player.init
pt3player.play  EQU pt3player+5
pt3player.mute  EQU pt3player+8

    INCBIN "IntroMusic_24576-35012.bin"

;------------------------------------------------------------------------------
; Intro entry point at 35013
; Jump to the main program
;------------------------------------------------------------------------------

    JP Main

;==============================================================================
; Static variables
;==============================================================================

;------------------------------------------------------------------------------
; Car Animation Data (CarAnimData)
; Contain dot position and displacement values
;------------------------------------------------------------------------------
; Description of data structure
;   bytes 0-7   = data of 1st dot
;   bytes 8-13  = data of 2nd dot
;   ...             ...
; Description of one dot data
;     offset            data
;     ------            ----
;       +0              horizontal coordinate (X)
;       +2              vertical coordinate (Y)
;       +4              horizontal displacement (DelX)
;       +6              vertical displacement (DelY)
;   All numerical values are 16-bit signed fixed point numbers in two's 
;   complement with 9-bit integer part (bits 15 to 7) and 7-bit fractional part
;   (bits 6-0). 
;------------------------------------------------------------------------------

CarAnimData
    INCBIN "morph_car1.bin"   ; Include Car1 and Car2 anim data

;==============================================================================
; Main program
;==============================================================================

Main
; Initialization
    HALT
    EXX                         ; Preserve HL'
    PUSH HL
;
    DI
    CALL pt3player.init         ; Initialize music player
    LD HL,IM2_ISR               ; Copy ISR to final destiny
    LD DE,IMER
    LD BC,ISR_size
    LDIR
    LD (Timer),BC               ; Reset timer variable
    LD A,#C3                    ; "JP" opcode
    LD (#FFF4),A
    LD HL,IMER                  ; Argument of JP instruction
    LD (#FFF5),HL
    LD A,#18                    ; "JR" opcode
    LD (#FFFF),A
    LD A,IMVEC.msb              ; 
    LD I,A
    IM 2
    EI
; Set backgroud color attributes and border to 0
    LD HL,22528
    LD (HL),L
    LD A,L
    OUT (254),A
    LD DE,22529
    LD BC,767
    LDIR

; 1st screen
    CALL PlotDots               ; Plot randomly distributed dots
    LD A,8                      ; Morph to Car1 in 8 steps
    CALL AnimDots
    CALL ClearAttrTop           ; Set attributes
    LD DE,16384
    LD HL,Screen.f1
    CALL dzx7_agilercs
    LD DE,192
    CALL Pause

; 2nd screen
    CALL ClearScrTop            ; Clear 1st title
    LD A,3                      ; Define anim steps (2^3=8)
    LD DE,+(Car.f3-Car.f2)/2    ; Size of object
    LD HL,Car.f2                ; Address of object
    CALL DisplCalc              ; Calculate displacement values
    LD A,8                      ; Animate morphing in 8 steps
    CALL AnimDots
    CALL ClearAttrBot           ; Set attributes
    LD DE,20480
    LD HL,Screen.f2
    CALL dzx7_agilercs
    LD DE,192*2
    CALL Pause

; 3rd screen
    CALL ClearScrBot            ; Clear 2nd title
    LD A,3                      ; Define anim steps (2^3=8)
    LD DE,+(Car.end-Car.f3)/2   ; Size of object
    LD HL,Car.f3                ; Address of object
    CALL DisplCalc              ; Calculate displacement values
    LD A,8                      ; Animate morphing in 8 steps
    CALL AnimDots
    CALL ClearAttrTop           ; Set attributes
    LD DE,16384
    LD HL,Screen.f3
    CALL dzx7_agilercs
    LD DE,192*3
    CALL Pause

; Final screen
    CALL ClearScrTop            ; Clear 3rd title
    LD A,8
    CALL ScaleUp

    LD DE,640
    CALL Pause

; End
IntroEnd

    POP HL                      ; Restore HL'
    EXX
    LD BC,(Timer)               ; Return timer value

;------------------------------------------------------------------------------
; Code for testing purposes, must be commented out
;    PUSH BC
;KWait
;    XOR A
;    IN A,(254)
;    RRCA
;    JR C,KWait
;    DI
;    CALL pt3player.mute
;    IM 1
;    EI
;    POP BC
; End testing code
;------------------------------------------------------------------------------

    RET                         ; Return to BASIC

;==============================================================================
; Subroutines
;==============================================================================

;------------------------------------------------------------------------------
; Interrupt service routine
; It'll be copied to IMER
;------------------------------------------------------------------------------

IM2_ISR
    PUSH AF                     ; Save register values
    PUSH BC
    PUSH DE
    PUSH HL
    PUSH IX
    EX AF,AF'
    PUSH AF
    LD HL,(Timer)               ; Increase timer value
    INC HL
    LD (Timer),HL
    CALL pt3player.play         ; Execute AY mod player
    POP AF                      ; Restore register values
    EX AF,AF'
    POP IX
    POP HL
    POP DE
    POP BC
    POP AF
    EI                          ; Turn on maskable interrupts
    RET                         ; End of ISR

ISR_size        EQU $-IM2_ISR   ; Size of ISR code
IMER            EQU #FFC0       ; Address of ISR code
Timer           EQU IMER+ISR_size
                                ; Timer variable is locate just after ISR code
IMVEC.msb       EQU #3B         ; MSB of IM2 vector held by I register

;------------------------------------------------------------------------------
; Clear Screen or Attributes
;------------------------------------------------------------------------------

ClearScrTop
; Clear lines 0-63 of screen (bitmap only)
    LD HL,16384                 ; Clear top third of sreen
ClearScrBot.j
    LD BC,2047                  ; Counter of bytes to clear
    LD (HL),L                   ; Clear first byte
ClearAttrTop.j
    LD E,L                      ; Calculate next byte address
    LD D,H
    INC DE
    LDIR                        ; Clear remaining bytes
    RET                         ; End of subroutine

; Clear lines 128-191 of screen (bitmap only)
ClearScrBot
    LD HL,20480                 ; Clear bottom third of screen
    JR ClearScrBot.j

; Set attributes of lines 0-63 of screen
ClearAttrTop
    LD HL,22528                 ; Clear top third of attribute area
ClearAttrBot.j
    LD BC,255                   ; Counter of bytes to clear
    LD (HL),%01000111           ; Brigh 1, paper 0, ink 7
    JR ClearAttrTop.j

; Set attributes of lines 128-191 of screen
ClearAttrBot
    LD HL,23040                 ; Clear bootm third of attribute area
    JR ClearAttrBot.j

;------------------------------------------------------------------------------
; Pause
;------------------------------------------------------------------------------

Pause
    LD HL,(Timer)
    OR A
    SBC HL,DE
    JR C,Pause
    RET

;------------------------------------------------------------------------------
; Object Conversion (ObjConv)
; Convert an object with 8-bit unsigned XY coordinates to 16-bit signed fixed
; point coordinates and store to anim data structure. If object has more dots
; than spaces in anim data structure, the extra space is filled with object
; coordinates.
;------------------------------------------------------------------------------
; Parameters:
;   HL=address of object to be converted
;   IX=address of anim data structure
;   DE=number of dots from object
; Output:
;   HL=address of object to be converted (unchanged)
;   IX=(end of anim data structure)+7
;   DE=dot number of object (unchanged)
;   A=0, BC=0
;------------------------------------------------------------------------------

ObjConv
    LD BC,AnimSize          ; Counter for number of dots
    PUSH HL                 ; Save object data pointer
    PUSH DE                 ; Save dot number of object
ObjConv.l0
    LD A,(HL)               ; Fetch 8-bit X coordinate
    INC HL                  ; Increment pointer to next coordinate
    LD (IX+0),0             ; Reset LSB of 16-bit coordinate
    SRL A                   ; The bit 0 of 8-bit coordinate is transferred to  
    RR (IX+0)               ;bit 7 of 16-bit's LSB
    LD (IX+1),A             ; Store bits 1-7 to 16-bit's MSB
    LD A,(HL)               ; Fetch 8-bit Y coordinate
    INC HL                  ; Increment pointer to next coordinate
    LD (IX+2),0             ; Reset LSB of 16-bit coordinate
    SRL A                   ; The bit 0 of 8-bit coordinate is transferred to  
    RR (IX+2)               ;bit 7 of 16-bit's LSB
    LD (IX+3),A             ; Store bits 1-7 to 16-bit's MSB
    PUSH DE
    LD DE,8
    ADD IX,DE               ; Advance pointer 8 positions
    POP DE
    DEC DE                  ; Decrement object dot counter
    LD A,E
    OR D
    JR NZ,ObjConv.s0        ; Skip next part if counter isn't 0
    POP DE                  ; Reset counter and pointer
    POP HL
    DEC SP                  ; Restore stack position to keep counter and 
    DEC SP                  ;pointer
    DEC SP
    DEC SP
ObjConv.s0
    DEC BC                  ; Decrement counter of anim data dots
    LD A,B
    OR C
    JR NZ,ObjConv.l0        ; Repeat until all dots are done
    POP DE                  ; Throw away data from stack
    POP HL
    RET                     ; End of subroutine
    
;------------------------------------------------------------------------------
; Displacement Calculation (DisplCalc)
; Given an object, calculate displacement values to change of dots from
; current configuration to the new one in 2^I steps. The number of steps must
; be equal to 2 raised by an integer number, in order to simplify division
; math operation.
;------------------------------------------------------------------------------
; Parameters:
;   A=base 2 logarithm of number of steps
;   DE=number of dots of object 
;   HL=address of object
;------------------------------------------------------------------------------

DisplCalc
    LD IX,CarAnimData           ; IX=address of anim data structure
    EX AF,AF'                   ; A'=log2 of number of anim steps
    LD BC,AnimSize              ; Counter for number of dots
    PUSH DE                     ; Save object's dot number (will be DE')
    PUSH HL                     ; Save address of object
    EXX
    POP HL                      ; HL is pointing to object
    DEC SP                      ; Keep HL at top of stack
    DEC SP
DisplCalc.l0
    LD D,(HL)                   ; Fetch new X coordinate from object array
    INC HL                      ; Advance pointer
    LD E,0                      ; Convert 8-bit unsigned value to 16-bit signed
    SRL D
    RR E    
    PUSH HL                     ; Save object pointer
    LD L,(IX+0)                 ; Fetch old X coordinate
    LD H,(IX+1)
    EX DE,HL                    ; DE=old coordinate, HL=new coordinate
    AND A
    SBC HL,DE                   ; Calculate distance from old X to new X
    EX AF,AF'                   ; Fetch anim step exponent
    LD B,A
    EX AF,AF'
DisplCalc.l1
    SRA H                       ; Divide distance by 2 keeping arithmetic signal
    RR L
    DJNZ DisplCalc.l1
    LD (IX+4),L                 ; Store X displacement value
    LD (IX+5),H
    POP HL                      ; Recover array pointer
    LD D,(HL)                   ; Fetch new Y coordinate from object array
    INC HL                      ; Advance pointer
    LD E,0                      ; Convert 8-bit unsigned value to 16-bit signed
    SRL D
    RR E    
    PUSH HL                     ; Save array pointer
    LD L,(IX+2)                 ; Fetch old Y coordinate
    LD H,(IX+3)
    EX DE,HL                    ; DE=old coordinate, HL=new coordinate
    AND A
    SBC HL,DE                   ; Calculate distance from old Y to new Y
    EX AF,AF'                   ; Fetch anim step exponent
    LD B,A
    EX AF,AF'
DisplCalc.l2
    SRA H                       ; Divide distance by 2 keeping arithmetic signal
    RR L
    DJNZ DisplCalc.l2
    LD (IX+6),L                 ; Store Y displacement value
    LD (IX+7),H
    POP HL                      ; Recover array pointer
    LD DE,8                     ; Advance pointer 8 bytes
    ADD IX,DE
    EXX
    DEC DE                      ; Decrement counter of object's dot
    LD A,E
    OR D
    JR NZ,DisplCalc.s0          ; Skip next part if not 0
    EXX
    POP HL                      ; Reset object pointer
    EXX
    POP DE                      ; Reset counter
    DEC SP                      ; Restore stack status
    DEC SP
    DEC SP
    DEC SP
DisplCalc.s0
    DEC BC                      ; Decrement dot counter of anim data
    LD A,C
    OR B
    EXX
    JR NZ,DisplCalc.l0          ; Repeat until all dots are done
    POP HL                      ; Throw away garbage from stack
    POP DE
    RET                         ; End of subroutine

;------------------------------------------------------------------------------
; Animate Dots (AnimDots)
; Animate all dots A frames. For each dot, erase old position, plot at new
; position and set color attribute.
;------------------------------------------------------------------------------
; Parameter:
;   A=number of animation steps
; Destroyed registers: AF, BC, DE, HL, IX, AF', BC', DE', HL'
;------------------------------------------------------------------------------

AnimDots
    LD HL,22528             ; Set all attribute area with car color
    LD (HL),CarColor        ; Bright 1, paper 0, ink 2
    LD DE,22529
    LD BC,767
    LDIR                    ; Set all attribute area with above value
AnimDots.l0
    EX AF,AF'               ; Save anim step counter
    LD IX,CarAnimData       ; Pointer for animation data
    LD BC,AnimSize
    LD DE,8                 ; DE' is the constant to be added to IX pointer 
AnimDots.l1
    EXX                     ; Save counter (BC') and constant (DE')
    LD E,(IX+0)             ; Fetch old X coordinate
    LD D,(IX+1)
    LD L,(IX+4)             ; Fetch DelX displacement value
    LD H,(IX+5)
    ADD HL,DE               ; Add DelX to X to calculate new coordinate
    LD (IX+0),L             ; Store new X coordinate
    LD (IX+1),H
    RL E                    ; Calculate integer part of old X coordinate
    RL D
    LD C,D                  ; Store in C
    RL L                    ; Calculate integer part of new X coordinate
    RL H
    LD A,H                  ; Store in A
    LD E,(IX+2)             ; Fetch old Y coordinate
    LD D,(IX+3)
    LD L,(IX+6)             ; Fetch DelY displacement value
    LD H,(IX+7)
    ADD HL,DE               ; Add DelY to Y to calculate new coordinate
    LD (IX+2),L             ; Store new Y coordinate
    LD (IX+3),H
    RL E                    ; Calculate integer part of old Y coordinate
    RL D
    LD B,D                  ; Store in B
    RL L                    ; Calculate integer part of new Y coordinate
    RL H
    LD L,A                  ; Now HL contains new XY coordinates
    PUSH HL                 ; Put new XY coordinates onto stack
    CALL VideoPixelAdd      ; Calculate address of pixel
    LD A,E                  ; Erase dot at old position
    CPL
    AND (HL)
    LD (HL),A
    POP BC                  ; Get new XY coordinates
    CALL VideoPixelAdd      ; Calculate address of pixel
    LD A,E                  ; Plot at new dot position
    OR (HL)
    LD (HL),A
    EXX                     ; Restore counter (BC') and constant (DE')
    ADD IX,DE               ; Advance pointer to next dot data (8 bytes)
    DEC BC                  ; Decrement dot counter
    LD A,B                  ; Test if result is 0
    OR C
    JR NZ,AnimDots.l1       ; Repeat until all dots were plotted

    EX AF,AF'               ; Recover anim step counter
    DEC A
    JR NZ,AnimDots.l0       ; Repeat until last anim step

;------------------------------------------------------------------------------
; PlotDots
; Plot dots contained in anim data structure.
;------------------------------------------------------------------------------
; Parameters
;------------------------------------------------------------------------------

PlotDots
    LD IX,CarAnimData       ; Pointer for animation data
    LD BC,AnimSize          ; Counter for number of dots
    LD DE,8                 ; DE' is the constant to be added to IX pointer 
PlotDots.l0
    EXX                     ; Save counter (BC') and constant (DE')
    LD A,(IX+0)             ; Fetch X coordinate
    LD C,(IX+1)
    RLA                     ; Calculate integer part of old X coordinate
    RL C
    LD A,(IX+2)             ; Fetch old Y coordinate
    LD B,(IX+3)
    RLA                     ; Calculate integer part of old Y coordinate
    RL B
    CALL VideoPixelAdd      ; Calculate address of pixel
    LD A,E                  ; Plot at new dot position
    OR (HL)
    LD (HL),A
    EXX                     ; Restore counter (BC') and constant (DE')
    ADD IX,DE               ; Advance pointer to next dot data (8 bytes)
    DEC BC                  ; Decremente dot counter
    LD A,B                  ; Test if result is 0
    OR C
    JP NZ,PlotDots.l0       ; Repeat until all dots were plotted
    RET                     ; End of subroutine

;------------------------------------------------------------------------------
; ScaleUp
;------------------------------------------------------------------------------

ScaleUp
    LD HL,22528             ; Set all attribute area with car color
    LD (HL),CarColor        ; Bright 1, paper 0, ink 2
    LD DE,22529
    LD BC,767
    LDIR                    ; Set all attribute area with above value
    LD A,8
ScaleUp.l0
    EX AF,AF'               ; Save anim step counter
    LD IX,CarAnimData       ; Pointer for animation data
    LD BC,AnimSize
    LD DE,8                 ; DE' is the constant to be added to IX pointer 
ScaleUp.l1
    EXX                     ; Save counter (BC') and constant (DE')

    LD E,(IX+2)             ; Fetch old Y coordinate
    LD D,(IX+3)
    LD A,D
    CP 192/2                ; Skip next part if Y is above limit of 191
    JR NC,ScaleUp.s0
    LD HL,-106*128          ; Y=1,25*Y-106/4
    ADD HL,DE
    SRA H
    RR L
    SRA H
    RR L
    ADD HL,DE
    LD (IX+2),L             ; Store new Y coordinate
    LD (IX+3),H
    RL E                    ; Calculate integer part of old Y coordinate
    RL D
    LD B,D                  ; Store in B
    RL L                    ; Calculate integer part of new Y coordinate
    RL H
    JR NC,ScaleUp.s1         ; Assign invalid value (i.e. >191) if Y is negative
    LD H,255
ScaleUp.s1
    LD A,H                  ; Store in A

    LD E,(IX+0)             ; Fetch old X coordinate
    LD D,(IX+1)
    LD HL,-165*128          ; X=1,25*X-165/4
    ADD HL,DE
    SRA H
    RR L
    SRA H
    RR L
    ADD HL,DE
    BIT 7,H
    JR Z,ScaleUp.s2         ; Assign invalid value to Y (i.e. >191) if X is 
    LD A,255                ;negative
    LD (IX+3),A
ScaleUp.s2
    LD (IX+0),L             ; Store new X coordinate
    LD (IX+1),H
    RL E                    ; Calculate integer part of old X coordinate
    RL D
    LD C,D                  ; Store in C
    RL L                    ; Calculate integer part of new X coordinate
    RL H
    LD L,H
    LD H,A                  ; HL=new XY coord1

    PUSH HL                 ; Put new XY coordinates onto stack
    CALL VideoPixelAdd      ; Calculate address of pixel
    LD A,E                  ; Erase dot at old position
    CPL
    AND (HL)
    LD (HL),A

    POP BC                  ; Get new XY coordinates
    LD A,B
    CP 192
    JR NC,ScaleUp.s0
    CALL VideoPixelAdd      ; Calculate address of pixel
    LD A,E                  ; Plot at new dot position
    OR (HL)
    LD (HL),A

ScaleUp.s0
    EXX                     ; Restore counter (BC') and constant (DE')
    ADD IX,DE               ; Advance pointer to next dot data (8 bytes)
    DEC BC                  ; Decrement dot counter
    LD A,B                  ; Test if result is 0
    OR C
    JR NZ,ScaleUp.l1        ; Repeat until all dots were plotted
    HALT
    EX AF,AF'               ; Recover anim step counter
    DEC A
    JP NZ,ScaleUp.l0        ; Repeat until last anim step
    RET

;------------------------------------------------------------------------------
; Video Pixel Address (VideoPixelAdd)
; This subroutine accepts as parameters, a pair of coordinates defining a pixel 
; position, and it returns the corresponding address at video RAM and 1-bit 
; mask with the position of pixel within the byte.
;------------------------------------------------------------------------------
; Parameters:
;       B=vertical coordinate  (0-191)
;       C=horizontal coordinate (0-255)
; Output:
;       HL=address at video RAM
;       E=1-bit mask of pixel for byte pointed by HL
;       B is unchanged (vertical coordinate)
;       C=byte column position (0-31)
;       A=H
;       Flags: C=0; Z=0; S=0
;------------------------------------------------------------------------------

    INCLUDE "VideoRAMAddress.asm"

; -----------------------------------------------------------------------------
; "Agile" integrated RCS+ZX7 decoder by Einar Saukas (150 bytes)
; -----------------------------------------------------------------------------
; Parameters:
;   HL: source address (compressed data)
;   DE: destination address (decompressing)
; -----------------------------------------------------------------------------
    INCLUDE "dzx7_agileRCS.asm"

;==============================================================================
; Data
;==============================================================================

;------------------------------------------------------------------------------
; Car
; Car.f1 and Car.f2 are precalculated and stored at CarAnimData variable
;------------------------------------------------------------------------------

Car.f2
    INCBIN "car2s.bin"
Car.f3
    INCBIN "car3s.bin"
Car.end

;------------------------------------------------------------------------------
; RCS encoded and ZX7 compressed screens
;------------------------------------------------------------------------------
Screen.f1
    INCBIN "00-intro1-title.bin.rcs.zx7"
Screen.f2
    INCBIN "00-intro2-title.bin.rcs.zx7"
Screen.f3
    INCBIN "00-intro3-title.bin.rcs.zx7"

;==============================================================================
; Created variables
;==============================================================================


;==============================================================================
    END
;==============================================================================
